@@ -1561,6 +1561,11 @@ extern int totempg_reconfigure (void)
 	return totemsrp_reconfigure (totemsrp_context, totempg_totem_config);
 }
 
+extern int totempg_crypto_reconfigure_phase (cfg_message_crypto_reconfig_phase_t phase)
+{
+	return totemsrp_crypto_reconfigure_phase (totemsrp_context, totempg_totem_config, phase);
+}
+
 extern void totempg_stats_clear (int flags)
 {
 	if (flags & TOTEMPG_STATS_CLEAR_TOTEM) {
@@ -1585,3 +1590,24 @@ void totempg_force_gather (void)
 {
 	totemsrp_force_gather(totemsrp_context);
 }
+
+/* Assumes ->orig_interfaces is already allocated */
+void totempg_get_config(struct totem_config *config)
+{
+	struct totem_interface *temp_if = config->orig_interfaces;
+
+	memcpy(config, totempg_totem_config, sizeof(struct totem_config));
+	config->orig_interfaces = temp_if;
+	memcpy(config->orig_interfaces, totempg_totem_config->interfaces, sizeof(struct totem_interface) * INTERFACE_MAX);
+	config->interfaces = NULL;
+}
+
+void totempg_put_config(struct totem_config *config)
+{
+	struct totem_interface *temp_if = totempg_totem_config->interfaces;
+
+	/* Preseve the existing interfaces[] array as transports might have pointers saved */
+	memcpy(totempg_totem_config->interfaces, config->interfaces, sizeof(struct totem_interface) * INTERFACE_MAX);
+	memcpy(totempg_totem_config, config, sizeof(struct totem_config));
+	totempg_totem_config->interfaces = temp_if;
+}
